package com.derschmale.away3d.headShading
{
	import away3d.lights.DirectionalLight;
	import away3d.materials.methods.SoftShadowMapMethod;

	import com.bit101.components.HSlider;

	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Vector3D;
	import flash.text.AntiAliasType;
	import flash.text.Font;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;

	public class ConfigPanel extends Sprite
	{
		[Embed(source="/../embeds/fonts/Uni0553.ttf", fontName="uni 05_53", advancedAntiAliasing="true", fontWeight="regular", fontStyle="normal", mimeType="application/x-font")]
		private static var UniFontAsset:Class;

		private var _uniFont : Font = new UniFontAsset();

		private var light : DirectionalLight;
		private var shadowMethod : SoftShadowMapMethod;

		private var lightArcSlider:HSlider;
		private var lightAzimuthSlider:HSlider;
		private var rangeSlider : HSlider;

		public function ConfigPanel(light : DirectionalLight, shadowMethod : SoftShadowMapMethod)
		{
			this.light = light;
			this.shadowMethod = shadowMethod;
			initBackground()
			initLightDirection();
			updateSliders();
		}

		private function initBackground():void
		{
			graphics.beginFill(0, .2);
			graphics.drawRect(0, 0, 250, 120);
			graphics.endFill();
		}

		private function updateSliders() : void
		{
			var direction : Vector3D = light.direction;
			lightAzimuthSlider.value = Math.PI / 2 - Math.acos(-direction.y / direction.length);
			lightArcSlider.value = Math.atan2(direction.z, direction.x);
			rangeSlider.value = shadowMethod.range;
		}

		private function initLightDirection():void
		{
			createLabel("Light direction:", 13);
			lightArcSlider = new HSlider(this, 130, 20, onLightDirectionChange);
			lightArcSlider.minimum = 0;
			lightArcSlider.maximum = Math.PI*2;

			createLabel("Light elevation:", 33);
			lightAzimuthSlider = new HSlider(this, 130, 40, onLightDirectionChange);
			lightAzimuthSlider.minimum = -Math.PI/2;
			lightAzimuthSlider.maximum = Math.PI/2;

			createLabel("Shadow range:", 53);
			rangeSlider = new HSlider(this, 130, 60, onRangeChange);
			rangeSlider.minimum = 1;
			rangeSlider.maximum = 10;
		}

		private function createLabel(text : String, y:int):TextField
		{
			var label : TextField = new TextField();
			label.x = 10;
			label.y = y;
			label.embedFonts = true;
			label.selectable = false;
			label.antiAliasType = AntiAliasType.ADVANCED;
			label.autoSize = TextFieldAutoSize.LEFT;
			label.defaultTextFormat = getTextFormat();
			label.text = text;
			addChild(label);
			return label;
		}

		private function onLightDirectionChange(event:Event):void
		{
			var azimuth : Number = Math.PI/2 - lightAzimuthSlider.value;
			var arc : Number = lightArcSlider.value;
			light.direction = new Vector3D(
					Math.sin(azimuth)*Math.cos(arc),
					-Math.cos(azimuth),
					Math.sin(azimuth)*Math.sin(arc)
			);
		}

		private function onRangeChange(event : Event) : void
		{
			shadowMethod.range = rangeSlider.value;
		}

		private function getTextFormat():TextFormat
		{
			return new TextFormat(_uniFont.fontName, 11, 0xdddddd);
		}
	}
}
