package com.derschmale.away3d.headShading
{
	import away3d.core.base.CompactSubGeometry;
	import away3d.core.base.SubMesh;
	import away3d.entities.Mesh;
	import away3d.events.AssetEvent;
	import away3d.library.AssetLibrary;
	import away3d.loaders.parsers.OBJParser;

	import com.derschmale.patterns.tasks.Task;

	import flash.utils.setTimeout;

	public class ParseMeshTask extends Task
	{
		[Embed(source="/../embeds/head.obj", mimeType="application/octet-stream")]
		private var HeadModel : Class;

		private var _headModel : Mesh;
		private var subMeshIndex : int;

		public function ParseMeshTask()
		{
		}

		public function get headModel() : Mesh
		{
			return _headModel;
		}

		override public function execute() : void
		{
			loadModel();
		}

		private function loadModel() : void
		{
			AssetLibrary.enableParser(OBJParser);
			AssetLibrary.addEventListener(AssetEvent.ASSET_COMPLETE, onAssetComplete);
			AssetLibrary.loadData(new HeadModel());
		}

		private function onAssetComplete(event : AssetEvent) : void
		{
			_headModel = event.asset as Mesh;
			if (!_headModel) return;

			for (var i : int = 0; i < _headModel.subMeshes.length; ++i)
				_headModel.subMeshes[i].material = null;

			subMeshIndex = 0;
//			updateCompletion(1, _headModel.subMeshes.length + 1);
//			setTimeout(processNextSubMesh, 30);
			notifyComplete();
		}

		// not used in this case
		private function processNextSubMesh() : void
		{
			calculateTangents(_headModel.subMeshes[subMeshIndex]);
			++subMeshIndex;
			updateCompletion(1 + subMeshIndex, _headModel.subMeshes.length + 1);

			if (subMeshIndex == _headModel.subMeshes.length)
				notifyComplete();
			else setTimeout(processNextSubMesh, 30)
		}

		/**
		 * A rather unorthodox way of calculating tangents, because the model is not uv-mapped seamlessly
		 */
		private function calculateTangents(subMesh : SubMesh) : void
		{
			var subGeom : CompactSubGeometry = CompactSubGeometry(subMesh.subGeometry);

			subGeom.autoDeriveVertexTangents = false;
			subGeom.autoDeriveVertexNormals = false;

			var data : Vector.<Number> = subGeom.vertexData;
			var len : int = data.length;
			var stride : int = subGeom.vertexStride;
			var normalOffset : int = subGeom.vertexNormalOffset;
			var tangentOffset : int = subGeom.vertexTangentOffset;
			var i : int = 0;
			var index : int;

			// tangent = cross(normal, Y-UP)
			do {
				index = i + normalOffset;
				var nx : Number = data[index];
				var nz : Number = data[uint(index + 2)];
				index = i + tangentOffset;
				var d : Number = 1/Math.sqrt(nx*nx+nz*nz);
				data[index] = nz*d;
				data[uint(index+1)] = 0;
				data[uint(index+2)] = -nx*d;
				i += stride;
			} while (i < len);

			subGeom.updateData(data);
		}
	}
}
